// Firmware for Blitz-o-shield Rev.1
// Firmwareversion: Rev.0
// Copyright F.R. 2019
// www.ramser-elektro.at
// Modified by Stefan Draeger - https://draeger-it.blog
//

#define ledRed 12
#define ledYellow 10
#define ledGreen 9
#define ledStrike 13

#define debug true

const int MAX_ANALOG_VALUE = 1023;
const int SECOND = 1000;

const word remoteImpulseTime = 000;  // Set how long the remote impulse will be

int pwmPin = 11;
int remotePin = 7;

int analogIn = A0;

word actValue;       // Actual value of analog signal
float averageValue;  // Averange value
int difference;      // Difference value (AverageValue - ActValue)

word strikeCount;      // Actual strikecounter. Every strike 32 is added.
word warnLevel = 255;  // Actual warning level
word decay;            // Value, how much the warnlevel is decreased, if no activity

word inactivityTimer;  // Timer for new PWM tune, if no activity

bool blink;  // Helper for LED blinking

unsigned long previousRemoteMillis = 0;  // Helper for remote impulse

word pwmDutyCycle;     // Actual duty cycle for PWM
word pwmDutyWatchdog;  // Helper for PWM Adjustment

unsigned long previousMillis = 0;  // Helper for 1 second cycle

void setup() {  // Assign the hardware setup
  Serial.begin(9600);

  pinMode(ledRed, OUTPUT);
  pinMode(ledYellow, OUTPUT);
  pinMode(ledGreen, OUTPUT);
  pinMode(ledStrike, OUTPUT);
  pinMode(pwmPin, OUTPUT);
  pinMode(remotePin, OUTPUT);

  analogReference(INTERNAL);  // Set the reference voltage to 1.1 volt
  tunePwm();
}

void tunePwm() {  // Setup the PWM and tune it
  if (debug) {
    Serial.println("PWM duty cycle tuning started");
  }
  pwmDutyCycle = 0;  // First discharge the capaciators
  analogWrite(pwmPin, pwmDutyCycle);
  digitalWrite(ledGreen, HIGH);  // Set green LED = capaciator discharged
  delay(250);

  actValue = analogRead(analogIn);  // Read dummy
  pwmDutyWatchdog = 0;              // Reset watchdog

  while (actValue < MAX_ANALOG_VALUE) {     // Has analogue input reached the maximum?
    actValue = analogRead(analogIn);        // Read analog input
    averageValue = actValue;                // Preset avarange value
    pwmDutyCycle = pwmDutyCycle + 1;        // Increase PWM level
    pwmDutyWatchdog = pwmDutyWatchdog + 1;  // Increase the PWM watchdog
    analogWrite(pwmPin, pwmDutyCycle);
    delay(250);                               // Always wait a little time, to load the capacitors
    if (actValue > (MAX_ANALOG_VALUE / 3)) {  // Set yellow LED = ActValue  > 1/3 of maximum.
      digitalWrite(ledYellow, HIGH);
    } else if (actValue > ((MAX_ANALOG_VALUE / 3) * 2)) {  // Set red LED = ActValue  > 2/3 of maximum.
      digitalWrite(ledRed, HIGH);
    }

    if (pwmDutyWatchdog >= 255) {  // Maximum PWM duty cycle reached. Something is wrong!!
      blink = !blink;              // LED Blink Helper
      if (debug) {
        Serial.println("Problem while adjust PWM Duty cycle. Maximum reached!!!");
      }
      digitalWrite(ledGreen, blink);  // Blink all LED in endless loop
      digitalWrite(ledYellow, blink);
      digitalWrite(ledRed, blink);
      digitalWrite(ledStrike, blink);
      delay(500);
    }
    actValue = analogRead(analogIn);  // Read analog input
    averageValue = actValue;          // Preset avarange value
  }

  pwmDutyCycle = (pwmDutyCycle / 3) * 2;
  analogWrite(pwmPin, pwmDutyCycle);
  digitalWrite(ledStrike, HIGH);  // Set white LED = Tune PWM successful
  delay(1000);
  digitalWrite(ledGreen, LOW);
  digitalWrite(ledYellow, LOW);
  digitalWrite(ledRed, LOW);
  digitalWrite(ledStrike, LOW);
  if (debug) {
    Serial.println("PWM duty cycle tune successfull");
  }
}

void loop() {
  actValue = analogRead(analogIn);   // Read in actual value
  averageValue = averageValue * 15;  // Calculate the averange value
  averageValue = averageValue + actValue;
  averageValue = averageValue / 16;
  difference = averageValue - actValue;

  if (difference >= 15) {  // STRIKE !!!
    digitalWrite(ledStrike, HIGH);
    digitalWrite(remotePin, HIGH);  // Set remote action
    previousRemoteMillis = millis();
    strikeCount = strikeCount + 32;
    if (strikeCount >= 255) {  //  More then 8 Strikes in 1 Second ?
      strikeCount = 255;       //  More then 8 strikes per second don't happen naturaly
    }
  } else {
    digitalWrite(ledStrike, LOW);
  }

  if ((millis() - previousRemoteMillis >= remoteImpulseTime) && (digitalRead(remotePin))) {
    // Remote impulse time elapsed
    digitalWrite(remotePin, LOW);
  }

  if (millis() - previousMillis >= SECOND) {  // 1 Second elapsed
    blink = !blink;                         // LED Blink Helper

    if (strikeCount > 32) {                 // At leased 2 strikes in one second
      warnLevel = warnLevel + strikeCount;  // Increase Warnlevel
    }

    decay = highByte(warnLevel);
    warnLevel = warnLevel - decay;  //  Level decreasing

    if (warnLevel < 2000) {  // No LED on
      digitalWrite(ledGreen, LOW);
      digitalWrite(ledYellow, LOW);
      digitalWrite(ledRed, LOW);
    } else if ((warnLevel >= 2000) and (warnLevel < 4000)) {  // Green LED on
      digitalWrite(ledGreen, blink);
      digitalWrite(ledYellow, LOW);
      digitalWrite(ledRed, LOW);
    } else if ((warnLevel >= 4000) && (warnLevel < 6000)) {  // Yellow LED on
      digitalWrite(ledGreen, LOW);
      digitalWrite(ledYellow, blink);
      digitalWrite(ledRed, LOW);
    } else if (warnLevel >= 6000) {  // Red LED on
      digitalWrite(ledGreen, LOW);
      digitalWrite(ledYellow, LOW);
      digitalWrite(ledRed, blink);
    } else if (warnLevel >= 7000) {  // Maximum reached
      warnLevel = 7000;
    }

    if (debug) {  // Send debugmessage over RS232
      // PWM;ActValue;AverangeValue;Difference;Warnlevel;Decay;Strikecount;
      Serial.print("PWM:");
      Serial.print(pwmDutyCycle);
      Serial.print("  Actual:");
      Serial.print(actValue);
      Serial.print("  Averange:");
      Serial.print(averageValue);
      Serial.print("  Difference:");
      Serial.print(difference);
      Serial.print("  Warnlevel:");
      Serial.print(warnLevel);
      Serial.print("  Decay:");
      Serial.print(decay);
      Serial.print("  Strikecount:");
      Serial.print(strikeCount);
      Serial.println();
    }

    if (decay == 0) {  // No activity. Increase InactivityTimer
      inactivityTimer = inactivityTimer + 1;
      if (inactivityTimer >= 3600) {  // No activity for one hour. Tune PWM.
        tunePwm();
        inactivityTimer = 0;
      }
    }

    strikeCount = 0;            // Reset strikecounter
    previousMillis = millis();  // Save actual millis, for the next 1 second cycle
  }
}
